import pytest


# Tests that:
# - leaf_fix gets set up first and torn down last
# - the two fix_concurrent_{1,2} fixtures run their setup/teardown code
#   at the same time -- their execution can be interleaved.
def test_fixture_basic_ordering(testdir):
    testdir.makepyfile(
        """
        import pytest
        from pytest_trio import trio_fixture
        from trio.testing import Sequencer

        setup_events = []
        teardown_events = []

        @trio_fixture
        def seq():
            return Sequencer()

        @pytest.fixture
        async def leaf_fix():
            setup_events.append("leaf_fix setup")
            yield
            teardown_events.append("leaf_fix teardown")

            assert teardown_events == [
                "fix_concurrent_1 teardown 1",
                "fix_concurrent_2 teardown 1",
                "fix_concurrent_1 teardown 2",
                "fix_concurrent_2 teardown 2",
                "leaf_fix teardown",
            ]

        @pytest.fixture
        async def fix_concurrent_1(leaf_fix, seq):
            async with seq(0):
                setup_events.append("fix_concurrent_1 setup 1")
            async with seq(2):
                setup_events.append("fix_concurrent_1 setup 2")
            yield
            async with seq(4):
                teardown_events.append("fix_concurrent_1 teardown 1")
            async with seq(6):
                teardown_events.append("fix_concurrent_1 teardown 2")

        @pytest.fixture
        async def fix_concurrent_2(leaf_fix, seq):
            async with seq(1):
                setup_events.append("fix_concurrent_2 setup 1")
            async with seq(3):
                setup_events.append("fix_concurrent_2 setup 2")
            yield
            async with seq(5):
                teardown_events.append("fix_concurrent_2 teardown 1")
            async with seq(7):
                teardown_events.append("fix_concurrent_2 teardown 2")

        @pytest.mark.trio
        async def test_root(fix_concurrent_1, fix_concurrent_2):
            assert setup_events == [
                "leaf_fix setup",
                "fix_concurrent_1 setup 1",
                "fix_concurrent_2 setup 1",
                "fix_concurrent_1 setup 2",
                "fix_concurrent_2 setup 2",
            ]
            assert teardown_events == []

        """
    )

    result = testdir.runpytest()
    result.assert_outcomes(passed=1)


def test_nursery_fixture_teardown_ordering(testdir):
    testdir.makepyfile(
        """
        import pytest
        from pytest_trio import trio_fixture
        import trio
        from trio.testing import wait_all_tasks_blocked

        events = []

        async def record_cancel(msg):
            try:
                await trio.sleep_forever()
            finally:
                events.append(msg)

        @pytest.fixture
        def fix0():
            yield
            assert events == [
                "test",
                "test cancel",
                "fix2 teardown",
                "fix2 cancel",
                "fix1 teardown",
                "fix1 cancel",
            ]

        @trio_fixture
        def fix1(nursery):
            nursery.start_soon(record_cancel, "fix1 cancel")
            yield
            events.append("fix1 teardown")

        @trio_fixture
        def fix2(fix1, nursery):
            nursery.start_soon(record_cancel, "fix2 cancel")
            yield
            events.append("fix2 teardown")

        @pytest.mark.trio
        async def test_root(fix2, nursery):
            nursery.start_soon(record_cancel, "test cancel")
            await wait_all_tasks_blocked()
            events.append("test")
        """
    )

    result = testdir.runpytest()
    result.assert_outcomes(passed=1)


def test_error_collection(testdir):
    # We want to make sure that pytest ultimately reports all the different
    # exceptions. We call .upper() on all the exceptions so that we have
    # tokens to look for in the output corresponding to each exception, where
    # those tokens don't appear at all the source (so we can't get a false
    # positive due to pytest printing out the source file).

    # We sleep at the beginning of all the fixtures b/c currently if any
    # fixture crashes, we skip setting up unrelated fixtures whose setup
    # hasn't even started yet. Maybe we shouldn't? But for now the sleeps make
    # sure that all the fixtures have started before any of them start
    # crashing.
    testdir.makepyfile(
        """
        import pytest
        from pytest_trio import trio_fixture
        import trio

        test_started = False

        @trio_fixture
        async def crash_nongen():
            with trio.CancelScope(shield=True):
                await trio.sleep(2)
            raise RuntimeError("crash_nongen".upper())

        @trio_fixture
        async def crash_early_agen():
            with trio.CancelScope(shield=True):
                await trio.sleep(2)
            raise RuntimeError("crash_early_agen".upper())
            yield

        @trio_fixture
        async def crash_late_agen():
            yield
            raise RuntimeError("crash_late_agen".upper())

        async def crash(when, token):
            with trio.CancelScope(shield=True):
                await trio.sleep(when)
                raise RuntimeError(token.upper())

        @trio_fixture
        def crash_background(nursery):
            nursery.start_soon(crash, 1, "crash_background_early")
            nursery.start_soon(crash, 3, "crash_background_late")

        @pytest.mark.trio
        async def test_all_the_crashes(
            autojump_clock,
            crash_nongen, crash_early_agen, crash_late_agen, crash_background,
        ):
            global test_started
            test_started = True

        def test_followup():
            assert not test_started

        """
    )

    result = testdir.runpytest()
    result.assert_outcomes(passed=1, failed=1)
    result.stdout.fnmatch_lines_random(
        [
            "*CRASH_NONGEN*",
            "*CRASH_EARLY_AGEN*",
            "*CRASH_LATE_AGEN*",
            "*CRASH_BACKGROUND_EARLY*",
            "*CRASH_BACKGROUND_LATE*",
        ]
    )


@pytest.mark.parametrize("bgmode", ["nursery fixture", "manual nursery"])
def test_background_crash_cancellation_propagation(bgmode, testdir):
    crashyfix_using_nursery_fixture = """
        @trio_fixture
        def crashyfix(nursery):
            nursery.start_soon(crashy)
            with pytest.raises(trio.Cancelled):
                yield
            # We should be cancelled here
            teardown_deadlines["crashyfix"] = trio.current_effective_deadline()
        """

    crashyfix_using_manual_nursery = """
        @trio_fixture
        async def crashyfix():
            async with trio.open_nursery() as nursery:
                nursery.start_soon(crashy)
                with pytest.raises(trio.Cancelled):
                    yield
                # We should be cancelled here
                teardown_deadlines["crashyfix"] = trio.current_effective_deadline()
        """

    if bgmode == "nursery fixture":
        crashyfix = crashyfix_using_nursery_fixture
    else:
        crashyfix = crashyfix_using_manual_nursery

    testdir.makepyfile(
        """
        import pytest
        from pytest_trio import trio_fixture
        import trio

        teardown_deadlines = {}
        final_time = None

        async def crashy():
            await trio.sleep(1)
            raise RuntimeError

        CRASHYFIX_HERE

        @trio_fixture
        def sidefix():
            yield
            # We should NOT be cancelled here
            teardown_deadlines["sidefix"] = trio.current_effective_deadline()

        @trio_fixture
        def userfix(crashyfix):
            yield
            # Currently we should NOT be cancelled here... though maybe this
            # should change?
            teardown_deadlines["userfix"] = trio.current_effective_deadline()

        @pytest.mark.trio
        async def test_it(userfix, sidefix, autojump_clock):
            try:
                await trio.sleep_forever()
            finally:
                global final_time
                final_time = trio.current_time()


        def test_post():
            assert teardown_deadlines == {
                "crashyfix": -float("inf"),
                "sidefix": float("inf"),
                "userfix": float("inf"),
            }
            assert final_time == 1
        """.replace(
            "CRASHYFIX_HERE", crashyfix
        )
    )

    result = testdir.runpytest()
    result.assert_outcomes(passed=1, failed=1)


# See the thread starting at
# https://github.com/python-trio/pytest-trio/pull/77#issuecomment-499979536
# for details on the real case that this was minimized from
def test_complex_cancel_interaction_regression(testdir):
    testdir.makepyfile(
        """
        import pytest
        import trio
        from contextlib import asynccontextmanager

        async def die_soon():
            raise RuntimeError('oops'.upper())

        @asynccontextmanager
        async def async_finalizer():
            try:
                yield
            finally:
                await trio.sleep(0)

        @pytest.fixture
        async def fixture(nursery):
            async with trio.open_nursery() as nursery1:
                async with async_finalizer():
                    async with trio.open_nursery() as nursery2:
                        nursery2.start_soon(die_soon)
                        yield
                        nursery1.cancel_scope.cancel()

        @pytest.mark.trio
        async def test_try(fixture):
            await trio.sleep_forever()
        """
    )

    result = testdir.runpytest()
    result.assert_outcomes(passed=0, failed=1)
    result.stdout.fnmatch_lines_random(["*OOPS*"])


# Makes sure that
# See https://github.com/python-trio/pytest-trio/issues/120
def test_fixtures_crash_and_hang_concurrently(testdir):
    testdir.makepyfile(
        """
        import trio
        import pytest


        @pytest.fixture
        async def hanging_fixture():
            print("hanging_fixture:start")
            await trio.Event().wait()
            yield
            print("hanging_fixture:end")


        @pytest.fixture
        async def exploding_fixture():
            print("exploding_fixture:start")
            raise Exception
            yield
            print("exploding_fixture:end")


        @pytest.mark.trio
        async def test_fails_right_away(exploding_fixture):
            ...


        @pytest.mark.trio
        async def test_fails_needs_some_scopes(exploding_fixture, hanging_fixture):
            ...
        """
    )

    result = testdir.runpytest()
    result.assert_outcomes(passed=0, failed=2)



import pytest
import trio


async def handle_client(stream):
    while True:
        buff = await stream.receive_some(4)
        await stream.send_all(buff)


@pytest.fixture
async def server(nursery):
    listeners = await nursery.start(trio.serve_tcp, handle_client, 0)
    return listeners[0]


@pytest.mark.trio
async def test_try(server):
    stream = await trio.testing.open_stream_to_socket_listener(server)
    await stream.send_all(b"ping")
    rep = await stream.receive_some(4)
    assert rep == b"ping"



import pytest


@pytest.fixture
def sync_fix():
    return "sync_fix"


@pytest.mark.trio
async def test_single_sync_fixture(sync_fix):
    assert sync_fix == "sync_fix"


def test_single_yield_fixture(testdir):

    testdir.makepyfile(
        """
        import pytest

        events = []

        @pytest.fixture
        def fix1():
            events.append('fixture setup')
            yield 'fix1'
            events.append('fixture teardown')

        def test_before():
            assert not events

        @pytest.mark.trio
        async def test_actual_test(fix1):
            assert events == ['fixture setup']
            assert fix1 == 'fix1'

        def test_after():
            assert events == [
                'fixture setup',
                'fixture teardown',
            ]
    """
    )

    result = testdir.runpytest()

    result.assert_outcomes(passed=3)


def test_single_yield_fixture_with_async_deps(testdir):

    testdir.makepyfile(
        """
        import pytest
        import trio

        events = []

        @pytest.fixture
        async def fix0():
            events.append('fix0 setup')
            await trio.sleep(0)
            return 'fix0'

        @pytest.fixture
        def fix1(fix0):
            events.append('fix1 setup')
            yield 'fix1 - ' + fix0
            events.append('fix1 teardown')

        def test_before():
            assert not events

        @pytest.mark.trio
        async def test_actual_test(fix1):
            assert events == ['fix0 setup', 'fix1 setup']
            assert fix1 == 'fix1 - fix0'

        def test_after():
            assert events == [
                'fix0 setup',
                'fix1 setup',
                'fix1 teardown',
            ]
    """
    )

    result = testdir.runpytest()

    result.assert_outcomes(passed=3)


def test_sync_yield_fixture_crashed_teardown_allow_other_teardowns(testdir):

    testdir.makepyfile(
        """
        import pytest
        import trio

        setup_events = set()
        teardown_events = set()

        @pytest.fixture
        async def force_async_fixture():
            pass

        @pytest.fixture
        def good_fixture(force_async_fixture):
            setup_events.add('good_fixture setup')
            yield
            teardown_events.add('good_fixture teardown')

        @pytest.fixture
        def bad_fixture(force_async_fixture):
            setup_events.add('bad_fixture setup')
            yield
            teardown_events.add('bad_fixture teardown')
            raise RuntimeError('Crash during fixture teardown')

        def test_before():
            assert not setup_events
            assert not teardown_events

        @pytest.mark.trio
        async def test_actual_test(bad_fixture, good_fixture):
            pass

        def test_after():
            assert setup_events == {
                'good_fixture setup',
                'bad_fixture setup',
            }
            assert teardown_events == {
                'bad_fixture teardown',
                'good_fixture teardown',
            }
    """
    )

    result = testdir.runpytest()

    result.assert_outcomes(failed=1, passed=2)
    result.stdout.re_match_lines(
        [r"(E\W+| +\| )RuntimeError: Crash during fixture teardown"]
    )



import pytest
import trio
from hypothesis import given, settings, strategies as st

from pytest_trio.plugin import _trio_test_runner_factory

# deadline=None avoids unpredictable warnings/errors when CI happens to be
# slow (example: https://travis-ci.org/python-trio/pytest-trio/jobs/406738296)
# max_examples=5 speeds things up a bit
our_settings = settings(deadline=None, max_examples=5)


@our_settings
@given(st.integers())
@pytest.mark.trio
async def test_mark_inner(n):
    assert isinstance(n, int)


@our_settings
@pytest.mark.trio
@given(st.integers())
async def test_mark_outer(n):
    assert isinstance(n, int)


@our_settings
@pytest.mark.parametrize("y", [1, 2])
@given(x=st.none())
@pytest.mark.trio
async def test_mark_and_parametrize(x, y):
    assert x is None
    assert y in (1, 2)


async def scheduler_trace():
    """Returns a scheduler-dependent value we can use to check determinism."""
    trace = []

    async def tracer(name):
        for i in range(10):
            trace.append((name, i))
            await trio.sleep(0)

    async with trio.open_nursery() as nursery:
        for i in range(5):
            nursery.start_soon(tracer, i)

    return tuple(trace)


def test_the_trio_scheduler_is_deterministic_under_hypothesis():
    traces = []

    @our_settings
    @given(st.integers())
    @pytest.mark.trio
    async def inner(_):
        traces.append(await scheduler_trace())

    # The pytest.mark.trio doesn't do it's magic thing to
    # inner functions, so we invoke it explicitly here.
    inner.hypothesis.inner_test = _trio_test_runner_factory(
        None, inner.hypothesis.inner_test
    )
    inner()  # Tada, now it's a sync function!

    assert len(traces) >= 5
    assert len(set(traces)) == 1



import pytest


def test_single_async_fixture(testdir):

    testdir.makepyfile(
        """
        import pytest
        import trio

        @pytest.fixture
        async def fix1():
            await trio.sleep(0)
            return 'fix1'

        @pytest.mark.trio
        async def test_simple(fix1):
            assert fix1 == 'fix1'
    """
    )

    result = testdir.runpytest()

    result.assert_outcomes(passed=1)


def test_async_fixture_recomputed_for_each_test(testdir):

    testdir.makepyfile(
        """
        import pytest
        import trio

        counter = 0

        @pytest.fixture
        async def fix1():
            global counter
            await trio.sleep(0)
            counter += 1
            return counter

        @pytest.mark.trio
        async def test_first(fix1):
            assert fix1 == 1

        @pytest.mark.trio
        async def test_second(fix1):
            assert fix1 == 2
    """
    )

    result = testdir.runpytest()

    result.assert_outcomes(passed=2)


def test_nested_async_fixture(testdir):

    testdir.makepyfile(
        """
        import pytest
        import trio

        @pytest.fixture
        async def fix1():
            await trio.sleep(0)
            return 'fix1'

        @pytest.fixture
        async def fix2(fix1):
            await trio.sleep(0)
            return 'fix2(%s)' % fix1

        @pytest.mark.trio
        async def test_simple(fix2):
            assert fix2 == 'fix2(fix1)'

        @pytest.mark.trio
        async def test_both(fix1, fix2):
            assert fix1 == 'fix1'
            assert fix2 == 'fix2(fix1)'
    """
    )

    result = testdir.runpytest()

    result.assert_outcomes(passed=2)


def test_async_within_sync_fixture(testdir):

    testdir.makepyfile(
        """
        import pytest
        import trio

        @pytest.fixture
        async def async_fix():
            await trio.sleep(0)
            return 42

        @pytest.fixture
        def sync_fix(async_fix):
            return async_fix

        @pytest.mark.trio
        async def test_simple(sync_fix):
            assert sync_fix == 42
    """
    )

    result = testdir.runpytest()

    result.assert_outcomes(passed=1)


# In pytest, ERROR status occurs when an exception is raised in fixture code.
# The trouble is our async fixtures must be run within a trio context, hence
# they are actually run just before the test, providing no way to make the
# difference between an exception coming from the real test or from an
# async fixture...
@pytest.mark.xfail(reason="Not implemented yet")
def test_raise_in_async_fixture_cause_pytest_error(testdir):

    testdir.makepyfile(
        """
        import pytest

        @pytest.fixture
        async def fix1():
            raise ValueError('Ouch !')

        @pytest.mark.trio
        async def test_base(fix1):
            pass  # Crash should have occurred before arriving here
    """
    )

    result = testdir.runpytest()

    result.assert_outcomes(errors=1)



import pytest
from pytest_trio import trio_fixture
import trio


@trio_fixture
def fixture_with_unique_name(nursery):
    nursery.start_soon(trio.sleep_forever)


@pytest.mark.trio
async def test_fixture_names(fixture_with_unique_name):
    # This might be a bit fragile ... if we rearrange the nursery hierarchy
    # somehow so it breaks, then we can make it more robust.
    task = trio.lowlevel.current_task()
    assert task.name == "<test 'test_fixture_names'>"
    sibling_names = {task.name for task in task.parent_nursery.child_tasks}
    assert "<fixture 'fixture_with_unique_name'>" in sibling_names



import pytest
from pytest_trio import trio_fixture

import contextvars

cv = contextvars.ContextVar("cv", default=None)


@trio_fixture
def cv_checker():
    assert cv.get() is None
    yield
    assert cv.get() is None


@trio_fixture
def cv_setter(cv_checker):
    assert cv.get() is None
    token = cv.set("cv_setter")
    yield
    assert cv.get() == "cv_setter2"
    cv.reset(token)
    assert cv.get() is None


@trio_fixture
def cv_setter2(cv_setter):
    assert cv.get() == "cv_setter"
    # Intentionally leak, so can check that this is visible back in cv_setter
    cv.set("cv_setter2")
    yield
    assert cv.get() == "cv_setter2"


@pytest.mark.trio
async def test_contextvars(cv_setter2):
    assert cv.get() == "cv_setter2"



def test_single_async_yield_fixture(testdir):
    testdir.makepyfile(
        """
        import pytest
        import trio

        events = []

        @pytest.fixture
        async def fix1():
            events.append('fix1 setup')
            await trio.sleep(0)

            yield 'fix1'

            await trio.sleep(0)
            events.append('fix1 teardown')

        def test_before():
            assert not events

        @pytest.mark.trio
        async def test_actual_test(fix1):
            assert events == ['fix1 setup']
            assert fix1 == 'fix1'

        def test_after():
            assert events == [
                'fix1 setup',
                'fix1 teardown',
            ]
    """
    )

    result = testdir.runpytest()

    result.assert_outcomes(passed=3)


def test_nested_async_yield_fixture(testdir):

    testdir.makepyfile(
        """
        import pytest
        import trio

        events = []

        @pytest.fixture
        async def fix2():
            events.append('fix2 setup')
            await trio.sleep(0)

            yield 'fix2'

            await trio.sleep(0)
            events.append('fix2 teardown')

        @pytest.fixture
        async def fix1(fix2):
            events.append('fix1 setup')
            await trio.sleep(0)

            yield 'fix1'

            await trio.sleep(0)
            events.append('fix1 teardown')

        def test_before():
            assert not events

        @pytest.mark.trio
        async def test_actual_test(fix1):
            assert events == [
                'fix2 setup',
                'fix1 setup',
            ]
            assert fix1 == 'fix1'

        def test_after():
            assert events == [
                'fix2 setup',
                'fix1 setup',
                'fix1 teardown',
                'fix2 teardown',
            ]
    """
    )

    result = testdir.runpytest()

    result.assert_outcomes(passed=3)


def test_async_yield_fixture_within_sync_fixture(testdir):

    testdir.makepyfile(
        """
        import pytest
        import trio

        events = []

        @pytest.fixture
        async def fix2():
            events.append('fix2 setup')
            await trio.sleep(0)

            yield 'fix2'

            await trio.sleep(0)
            events.append('fix2 teardown')

        @pytest.fixture
        def fix1(fix2):
            return 'fix1'

        def test_before():
            assert not events

        @pytest.mark.trio
        async def test_actual_test(fix1):
            assert events == [
                'fix2 setup',
            ]
            assert fix1 == 'fix1'

        def test_after():
            assert events == [
                'fix2 setup',
                'fix2 teardown',
            ]
    """
    )

    result = testdir.runpytest()

    result.assert_outcomes(passed=3)


def test_async_yield_fixture_within_sync_yield_fixture(testdir):

    testdir.makepyfile(
        """
        import pytest
        import trio

        events = []

        @pytest.fixture
        async def fix2():
            events.append('fix2 setup')
            await trio.sleep(0)

            yield 'fix2'

            await trio.sleep(0)
            events.append('fix2 teardown')

        @pytest.fixture
        def fix1(fix2):
            events.append('fix1 setup')
            yield 'fix1'
            events.append('fix1 teardown')

        def test_before():
            assert not events

        @pytest.mark.trio
        async def test_actual_test(fix1):
            assert events == [
                'fix2 setup',
                'fix1 setup',
            ]
            assert fix1 == 'fix1'

        def test_after():
            assert events == [
                'fix2 setup',
                'fix1 setup',
                'fix1 teardown',
                'fix2 teardown',
            ]
    """
    )

    result = testdir.runpytest()

    result.assert_outcomes(passed=3)


def test_async_yield_fixture_with_multiple_yields(testdir):

    testdir.makepyfile(
        """
        import pytest
        import trio

        @pytest.fixture
        async def fix1():
            await trio.sleep(0)
            yield 'good'
            await trio.sleep(0)
            yield 'bad'

        @pytest.mark.trio
        async def test_actual_test(fix1):
            pass
    """
    )

    result = testdir.runpytest()

    # TODO: should trigger error instead of failure
    # result.assert_outcomes(errors=1)
    result.assert_outcomes(failed=1)


def test_async_yield_fixture_with_nursery(testdir):

    testdir.makepyfile(
        """
        import pytest
        import trio


        async def handle_client(stream):
            while True:
                buff = await stream.receive_some(4)
                await stream.send_all(buff)


        @pytest.fixture
        async def server():
            async with trio.open_nursery() as nursery:
                listeners = await nursery.start(trio.serve_tcp, handle_client, 0)
                yield listeners[0]
                nursery.cancel_scope.cancel()


        @pytest.mark.trio
        async def test_actual_test(server):
            stream = await trio.testing.open_stream_to_socket_listener(server)
            await stream.send_all(b'ping')
            rep = await stream.receive_some(4)
            assert rep == b'ping'
    """
    )

    result = testdir.runpytest()

    result.assert_outcomes(passed=1)


def test_async_yield_fixture_crashed_teardown_allow_other_teardowns(testdir):

    testdir.makepyfile(
        """
        import pytest
        import trio

        setup_events = set()
        teardown_events = set()

        @pytest.fixture
        async def good_fixture():
            async with trio.open_nursery() as nursery:
                setup_events.add('good_fixture setup')
                yield None
                teardown_events.add('good_fixture teardown')

        @pytest.fixture
        async def bad_fixture():
            async with trio.open_nursery() as nursery:
                setup_events.add('bad_fixture setup')
                yield None
                teardown_events.add('bad_fixture teardown')
                raise RuntimeError('Crash during fixture teardown')

        def test_before():
            assert not setup_events
            assert not teardown_events

        @pytest.mark.trio
        async def test_actual_test(bad_fixture, good_fixture):
            pass

        def test_after():
            assert setup_events == {
                'good_fixture setup',
                'bad_fixture setup',
            }
            assert teardown_events == {
                'bad_fixture teardown',
                'good_fixture teardown',
            }
    """
    )

    result = testdir.runpytest()

    result.assert_outcomes(failed=1, passed=2)
    result.stdout.re_match_lines(
        [r"(E\W+| +\| )RuntimeError: Crash during fixture teardown"]
    )



import pytest


def enable_trio_mode_via_pytest_ini(testdir):
    testdir.makefile(".ini", pytest="[pytest]\ntrio_mode = true\n")


def enable_trio_mode_trio_run_via_pytest_ini(testdir):
    testdir.makefile(".ini", pytest="[pytest]\ntrio_mode = true\ntrio_run = trio\n")


def enable_trio_mode_via_conftest_py(testdir):
    testdir.makeconftest("from pytest_trio.enable_trio_mode import *")


enable_trio_mode = pytest.mark.parametrize(
    "enable_trio_mode",
    [
        enable_trio_mode_via_pytest_ini,
        enable_trio_mode_trio_run_via_pytest_ini,
        enable_trio_mode_via_conftest_py,
    ],
)
